<?php
namespace Teknisa\EST\Controllers;

use Geral\Entities\Retirada;
use Fiscal\Entities\Notafiscal;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\DateUtil;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\Util\Functions;

class FecharDia {
    protected $crudService;
    protected $fecharDiaService;
    protected $paramFilialService;

    protected $NRORG;
    protected $CDOPERADOR;

    protected $entityManager;
    protected $dataSourceManager;

    protected $errorMessage = "";

    /***************************************************************************
     * \/\/\/\/\/
     * ATENÇÃO!!!
     * /\/\/\/\/\
     * Esse controller e seus services se encontram fora do padrão MVC. Para
     * quaisquer alterações que vierem a serem feitas nos arquivos da tela Fechar
     * Dia, favor seguir os padrões de MVC.
     ***************************************************************************/
    
    public function __construct($crudService, Environment $environment, \Teknisa\EST\Services\Paramfilial $paramFilialService, \Teknisa\EST\Services\FecharDia $fecharDiaService, $dataSourceManager, $entityManager) {
        $this->crudService        = $crudService;
        $this->fecharDiaService   = $fecharDiaService;
        $this->paramFilialService = $paramFilialService;
        $this->environment        = $environment;

        $this->NRORG              = $environment->getNrOrgTrab();
        $this->CDOPERADOR         = $environment->getCdOperador();

        $this->entityManager      = $entityManager;
        $this->dataSourceManager  = $dataSourceManager;
    }

    # ESSA FUNÇÃO DEVERÁ RECEBER UM PARÂMETRO ADICIONAL QUE A DIRÁ SE DEVE
    # VALIDAR O ESTOQUE CEGO OU NÃO. A FUNCIONALIDADE REAL DESSE PARÂMETRO
    # FAR-SE-Á ÓBVIA QUANDO HOUVER RETORNO PARA O FRONTEND PARA PEDIR
    # CONFIRMAÇÃO DA CONTINUAÇÃO DAS OPERAÇÕES PARA O USUÁRIO.

    public function validaFechaDia(\Zeedhi\Framework\DTO\Request\Filter $request, \Zeedhi\Framework\DTO\Response $response) {
        $requestConditions = $request->getFilterCriteria()->getConditions();


        $CDFILIAL       = $requestConditions[0]["value"];
        $DTPROCESSA     = $requestConditions[1]["value"];
        $proxDTPROCESSA = $requestConditions[2]["value"];
        $boVerifInvCego = $requestConditions[3]["value"]; //INDICA SE O INVENTÁRIO CEGO JÁ FOI VERIFICADO COM SUCESSO

        $params = $this->fecharDiaService->requestParameters($CDFILIAL);

        $haveError = false;

        $datasetValue = array();
        $datasetValue['CDFILIAL'] = $CDFILIAL;
        $datasetValue['DTPROCESSA'] = $DTPROCESSA;
        $datasetValue['PROXDTPROCESSA'] = $proxDTPROCESSA;

        $this->entityManager->beginTransaction();

        try {
            if(!$boVerifInvCego) {
                $stInventCego = $this->fecharDiaService->divergenciaEstoqueCego($CDFILIAL, $DTPROCESSA);

                if($stInventCego == 'S') {
                    throw new \Exception("Existe divergência de inventário pendente para a filial. Para realizar Aceite/Recusa, favor fechar o módulo e abrir novamente. ");
                }

                $vdEstCego = $this->fecharDiaService->vdEstCego($CDFILIAL, $DTPROCESSA, $proxDTPROCESSA);
                $inventarios = $vdEstCego["inventarios"];
                $stDivergencia = $vdEstCego["stDivergencia"];

                if(!empty($inventarios)) {
                    $problematicSituations = array("PR" => "programado", "DG" => "digitado", "ED" => "em digitação");
                    foreach($inventarios as $inventario){
                        if(Functions::arrayKeyExists($inventario["IDSITESTCEGO"], $problematicSituations)) {
                            if($inventario["IDBLOQFECINVPE"] == "N") {
                                $datasetValue['userprompt'] = true;
    
                                $response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('fechardia', $datasetValue));
                                $response->addMessage(new Message("Atenção! Existe inventário com a situação ".$problematicSituations[$inventario["IDSITESTCEGO"]].". Confirma operação?", "success"));
    
                                $this->entityManager->commit();
                                return;
                            } else
                                throw new \Exception("Existe inventário com a situação ".$problematicSituations[$inventario["IDSITESTCEGO"]].". ");
                        }
                    }
                }
            }
            $this->fecharDiaService->verificaConsolidacaoDeVendas($params["IDCONSISTECONSO"], $CDFILIAL, $DTPROCESSA);
            $this->fecharDiaService->CheckEffectiveDays($CDFILIAL, $DTPROCESSA);
            $this->fecharDiaService->verificaNFSaida($params['IDBLFECHSNFE'], $CDFILIAL, $DTPROCESSA);
            
        } catch(\Exception $e) {
            $this->errorMessage .= $e->getMessage();
            $haveError = true;
        }

        if($params['IDBLOQESTNEG'] == 'S') {
            try {
                $this->validaProdutoIrreg($CDFILIAL, $DTPROCESSA);
            } catch(\Exception $e) {
                $this->errorMessage .= $e->getMessage();
                $haveError = true;
            }
        }

        if($params['IDBLOQFECDIA'] == 'S') {
            try {
                $this->validaDtFechaMes($CDFILIAL, $DTPROCESSA);
            } catch(\Exception $e) {
                $this->errorMessage .= $e->getMessage();
                $haveError = true;
            }
        }
        
        if($params['IDBLOQAJUSNEVOL'] == 'S'){
            try {
                $this->validaImplatacaoAjusteDesatualizado($CDFILIAL, $DTPROCESSA);
            } catch(\Exception $e) {
                $this->errorMessage .= $e->getMessage();
                $haveError = true;
            }
        }

        if($params['IDUTILALMOX'] == 'S' && $params['IDINVEROTREQ'] == 'S' && is_null($params['CDALMOXARIFE'])) {
            $this->errorMessage .= "Almoxarifado padrão deve ser informado. ";
            $haveError = true;
        }

        if($params['IDUTILLCESTQ'] == 'S' && $params['IDINVEROTREQ'] == 'S' && is_null($params['CDLOCALESTOQ'])) {
            $this->errorMessage .= "Localização padrão deve ser informada. ";
            $haveError = true;
        }

        $inventError = false;

        if($params['IDUTILLOTE'] == 'N') {
            if(!$this->validaInventario($CDFILIAL, $DTPROCESSA)) {
                $haveError = true;
                $inventError = true;
            } else {
                $inverotaLote = false;
            }
        } else {
            if(!$this->validaInventariolote($CDFILIAL, $DTPROCESSA)) {
                $haveError = true;
                $inventError = true;
            } else {
                $inverotaLote = true;
            }
        }

        if($inventError) {
            if($params['IDUTILALMOX'] == 'S') {
                $inventErrorCodes = $this->fecharDiaService->requestInventErrorCodes($CDFILIAL, $DTPROCESSA, $params['IDUTILLOTE']);

                $this->errorMessage .= "Inventário pendente para o almoxarifado " . $inventErrorCodes["CDALMOXARIFE"];
                $this->errorMessage .= ($params['IDUTILLCESTQ'] == "S")? (" e localização " . $inventErrorCodes["CDLOCALESTOQ"] . ". ") : ". ";
            } else {
                $this->errorMessage .= "Inventário pendente para a unidade. ";
            }
        }

        if(!$haveError) {
            try {
                $this->fecharDiaService->verificaReSaidaEntr($CDFILIAL, $DTPROCESSA, $this->NRORG);
                
                if($params['USR_IDINTEGRASAP']=='S'){
                    $this->fecharDiaService->processCompass($CDFILIAL, $DTPROCESSA, $proxDTPROCESSA, $params['NRDIAFECHMES'], $params['IDGRAVATITABELA'], $this->CDOPERADOR, $this->NRORG);
                }
                
                $this->fecharDiaService->processGr($CDFILIAL, $DTPROCESSA, $proxDTPROCESSA, $params['NRDIAFECHMES'], $params['IDGRAVATITABELA'], $this->CDOPERADOR, $this->NRORG);
  
                $this->fecharDiaService->exportaSAP($CDFILIAL, $DTPROCESSA, $params['IDEXPINFSAP']);

                $DTINVEROTA = DateUtil::adicionaIntervalo(DateUtil::getDataDeString($DTPROCESSA))->format(DateUtil::FORMATO_BRASILEIRO);

                if(!$inverotaLote) {
                    $this->fecharDiaService->deleteInveRota($CDFILIAL, $this->NRORG);

                    $inventProg = false;
                    if($params['IDINVROTDIA'] == 'S') {
                       $inventProg = true;
                    } else {
                        $diaProcessa   = substr($DTPROCESSA, 0, 2);
                        $diaProgramado = $this->fecharDiaService->verificaInventProg($CDFILIAL, $diaProcessa, $this->NRORG);

                        foreach($diaProgramado as $item) {
                            if($item["NRDIAINVFILI"] == $diaProcessa) {
                                $inventProg = true;
                            }
                        }
                    }

                    if($inventProg) {
                        if($params['IDUTILALMOX'] == 'N') {
                            $produtosInveRota = $this->fecharDiaService->produtosInventProg($CDFILIAL, $DTINVEROTA, ' ', ' ', $this->NRORG);

                            foreach($produtosInveRota as $item) {
                                $this->fecharDiaService->saveItemInverota($CDFILIAL, $DTPROCESSA, $item['CDPRODUTO'], $item['QTESTOQDIA'], ' ', ' ', 0, $this->NRORG);
                            }

                            $datasetValue['InveRota'] = true;
                        } else if($params['IDUTILLCESTQ'] == 'N') {
                            $produtosInveRota = $this->fecharDiaService->produtosInventProgAlmoxarife($CDFILIAL, $DTINVEROTA, ' ', $this->NRORG);

                            foreach($produtosInveRota as $item) {
                                $this->fecharDiaService->saveItemInverota($CDFILIAL, $DTPROCESSA, $item['CDPRODUTO'], $item['QTESTOQDIA'], $item['CDALMOXARIFE'], ' ', 0, $this->NRORG);
                            }

                            $datasetValue['InveRota'] = true;
                        } else {
                            $produtosInveRota = $this->fecharDiaService->produtosInventProgAlmoLocal($CDFILIAL, $DTINVEROTA, $this->NRORG);

                            foreach($produtosInveRota as $item) {
                                $this->fecharDiaService->saveItemInverota($CDFILIAL, $DTPROCESSA, $item['CDPRODUTO'], $item['QTESTOQDIA'], $item['CDALMOXARIFE'], $item['CDLOCALESTOQ'], 0, $this->NRORG);
                            }

                            $datasetValue['InveRota'] = true;
                        }
                    } else {
                        $datasetValue['InveRota'] = false;
                    }
                } else {
                    $this->fecharDiaService->deleteInveRotaLote($CDFILIAL, $this->NRORG);
                    $inventProg = false;

                    if($params['IDINVROTDIA'] == 'S') {
                       $inventProg = true;
                    } else {
                        $diaProcessa   = substr($DTPROCESSA,0,2);
                        $diaProgramado = $this->fecharDiaService->verificaInventProg($CDFILIAL, $diaProcessa, $this->NRORG);

                        foreach($diaProgramado as $item) {
                            if($item["NRDIAINVFILI"] == $diaProcessa) {
                                $inventProg = true;
                            }
                        }
                    }

                    if($inventProg) {
                        if($params['IDUTILALMOX'] == 'N') {
                            $produtosInveRota = $this->fecharDiaService->produtosInventProg($CDFILIAL, $DTINVEROTA, ' ', ' ', $this->NRORG);

                            foreach($produtosInveRota as $item) {
                                $this->fecharDiaService->saveItemInverotaLote($CDFILIAL, $DTPROCESSA, $item['CDPRODUTO'], ' ', ' ', $this->NRORG, $item['NRLOTEESTQ'], $item['NRSUBLOTE'], 0, 0);
                            }

                            $datasetValue['InveRota'] = true;
                        } else if($params['IDUTILLCESTQ'] == 'N') {
                            $produtosInveRota = $this->fecharDiaService->produtosInventProgAlmoxarife($CDFILIAL, $DTINVEROTA, ' ', $this->NRORG);

                            foreach($produtosInveRota as $item) {
                                $this->fecharDiaService->saveItemInverotaLote($CDFILIAL, $DTPROCESSA, $item['CDPRODUTO'], $item['CDALMOXARIFE'], ' ', $this->NRORG, $item['NRLOTEESTQ'], $item['NRSUBLOTE'], 0, 0);
                            }

                            $datasetValue['InveRota'] = true;
                        } else {
                            $produtosInveRota = $this->fecharDiaService->produtosInventProgAlmoLocal($CDFILIAL, $DTINVEROTA, $this->NRORG);

                            foreach($produtosInveRota as $item) {
                                $this->fecharDiaService->saveItemInverotaLote($CDFILIAL, $DTPROCESSA, $item['CDPRODUTO'], $item['CDALMOXARIFE'], $item['CDLOCALESTOQ'], $this->NRORG, $item['NRLOTEESTQ'], $item['NRSUBLOTE'], 0, 0);
                            }

                            $datasetValue['InveRota'] = true;
                        }
                    } else {
                        $datasetValue['InveRota'] = false;
                    }
                }

                $this->paramFilialService->fechaDia($CDFILIAL, $proxDTPROCESSA);

                $datasetValue['userprompt'] = false;

                $this->entityManager->commit();

                $response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('fechardia', $datasetValue));
                $response->addMessage(new Message("Dia fechado com sucesso! ", "success"));
            } catch(\Exception $e) {
                $this->entityManager->rollback();

                $response->setError(new Error("Ocorreu um erro ao fechar dia. ".$e->getMessage(),"error"));
            }
        } else {
            $this->entityManager->rollback();

            if($this->errorMessage) {
                $this->errorMessage = 'Operação bloqueada! <br>' . $this->errorMessage;
                $response->setError(new Error($this->errorMessage, 500));
            }
        }
    }

    public function getDadosFecharDia(DTO\Request\Row $request, DTO\Response $response) {
        $CDFILIAL = ($this->environment->getCdfilial()) ? $this->environment->getCdfilial() : $row["CDFILIAL"];
        $dadosFecharDia = $this->fecharDiaService->getDadosFecharDia($CDFILIAL);
        $response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('dados_fechardia', $dadosFecharDia));
    }

    public function validaNFEntrada($CDFILIAL, $DTPROCESSA) {
        $this->parameterBagColumns = array(
          "P_CDFILIAL",
          "P_DTPROCESSA"
        );

        $filterCriteria = new \Zeedhi\Framework\DataSource\FilterCriteria("notafiscal_selectminusnotaitemnota");
        $filterCriteria->addCondition('P_CDFILIAL', $CDFILIAL);
        $filterCriteria->addCondition('P_DTPROCESSA', $DTPROCESSA);

        $newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, $this->parameterBagColumns);

        $dataset = $this->dataSourceManager->findBy($newFilterCriteria);
        $NF      = $dataset->getRows();
        
        if(!empty($NF)) {
            throw new \Exception("Existe entrada sem nota fiscal que não possui nota fiscal atrasada. ", 1);
        }
    }

    public function validaProdutoIrreg($CDFILIAL, $DTPROCESSA) {
        $this->parameterBagColumns = array(
          "P_CDFILIAL",
          "P_DTFINAL"
        );

        $mes = date("m", strtotime(str_replace("/", "-", $DTPROCESSA)));
        $ano = date("Y", strtotime(str_replace("/", "-", $DTPROCESSA)));

        $ultimo_dia = date("t", mktime(0,0,0,$mes,'01',$ano));
        
        $DTFINAL = (substr($DTPROCESSA, 0, 2) + 1).substr($DTPROCESSA, 2, 8);

        if(substr($DTPROCESSA, 0, 2) >= $ultimo_dia) {
            if($mes != '12') $mes = ((int) $mes) + 1;
            else {
                $mes = '01';
            }
            if(strlen($mes) == 1) $mes = '0'. $mes;
            $DTFINAL = '01/'.$mes."/$ano";
        }
        else {
            $DTFINAL = (substr($DTPROCESSA, 0, 2) + 1).substr($DTPROCESSA, 2, 8);
        }

        $filterCriteria = new \Zeedhi\Framework\DataSource\FilterCriteria("posestdia_validacomposestdia");
        $filterCriteria->addCondition('P_CDFILIAL', $CDFILIAL);
        $filterCriteria->addCondition('P_DTFINAL', $DTFINAL);

        $newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, $this->parameterBagColumns);

        $dataset = $this->dataSourceManager->findBy($newFilterCriteria);
        $produto = $dataset->getRows();

        if(isset($produto[0])) {
          throw new \Exception("Não é possível realizar o fechamento do dia com produtos irregulares. ", 1);
        }
    }

    public function validaDtFechaMes($CDFILIAL, $DTPROCESSA) {
        $this->parameterBagColumns = array(
          "P_CDFILIAL",
          "P_DTINICIAL",
          "P_DTFINAL"
        );

        $DTINICIAL = date('d/m/Y', mktime(0, 0, 0, substr($DTPROCESSA,strpos($DTPROCESSA,'/')+1,2), 1, substr($DTPROCESSA,strripos($DTPROCESSA,'/')+1,4)));
        $DTFINAL   = date('t/m/Y', mktime(0, 0, 0, substr($DTPROCESSA,strpos($DTPROCESSA,'/')+1,2), 1, substr($DTPROCESSA,strripos($DTPROCESSA,'/')+1,4)));

        $filterCriteria = new \Zeedhi\Framework\DataSource\FilterCriteria("diafecmes_betweendtfecmes");
        $filterCriteria->addCondition('P_CDFILIAL', $CDFILIAL);
        $filterCriteria->addCondition('P_DTINICIAL', $DTINICIAL);
        $filterCriteria->addCondition('P_DTFINAL', $DTFINAL);

        $newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, $this->parameterBagColumns);

        $dataset    = $this->dataSourceManager->findBy($newFilterCriteria);
        $dtFechaMes = $dataset->getRows();

        if(empty($dtFechaMes)) {
            throw new \Exception("Dia para fechamento do mês deve ser informado. ", 1);
        }
    }

    public function validaInventario($CDFILIAL, $DTPROCESSA) {
        $this->parameterBagColumns = array(
          "P_CDFILIAL",
          "P_DTINVEROTA"
        );

        $filterCriteria = new \Zeedhi\Framework\DataSource\FilterCriteria("inverota_validacominverota");
        $filterCriteria->addCondition('P_CDFILIAL', $CDFILIAL);
        $filterCriteria->addCondition('P_DTINVEROTA', $DTPROCESSA);

        $newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, $this->parameterBagColumns);

        $dataset    = $this->dataSourceManager->findBy($newFilterCriteria);
        $inventario = $dataset->getRows();

        if(!empty($inventario)) {
            return false;
        }

        return true;
    }

    public function validaInventarioLote($CDFILIAL, $DTPROCESSA) {
        $this->parameterBagColumns = array(
          "P_CDFILIAL",
          "P_DTINVEROTA"
        );

        $filterCriteria = new \Zeedhi\Framework\DataSource\FilterCriteria("inverotalote_validacominverotalote");
        $filterCriteria->addCondition('P_CDFILIAL', $CDFILIAL);
        $filterCriteria->addCondition('P_DTINVEROTA', $DTPROCESSA);

        $newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, $this->parameterBagColumns);

        $dataset    = $this->dataSourceManager->findBy($newFilterCriteria);
        $inventario = $dataset->getRows();

        if(!empty($inventario)) {
            return false;
        }

        return true;
    }
    
    public function validaImplatacaoAjusteDesatualizado($CDFILIAL, $DTPROCESSA){
        $this->fecharDiaService->validaImplatacaoAjusteDesatualizado($CDFILIAL, $DTPROCESSA);
    }
}